//@author: vux
//@help: Effect processing for skinned mesh with directional light.
//@tags: skinning, shading, blinn
//@credits: vvvv group

struct vsInput
{
    float4 posObject : POSITION;
	float4 normalObject : NORMAL;
};

struct psInput
{
    float4 posScreen : SV_Position;
    float3 LightDirV: TEXCOORD0;
    float3 NormV: TEXCOORD1;
    float3 ViewDirV: TEXCOORD2;
};

struct vsInputTextured
{
    float4 posObject : POSITION;
	float4 normalObject : NORMAL;
	float4 uv: TEXCOORD0;
};

struct psInputTextured
{
    float4 posScreen : SV_Position;
    float4 uv: TEXCOORD0;
    float3 LightDirV: TEXCOORD1;
    float3 NormV: TEXCOORD2;
    float3 ViewDirV: TEXCOORD3;
};

Texture2D inputTexture <string uiname="Texture";>;
StructuredBuffer<float4x4> JointMatrices;

SamplerState linearSampler <string uiname="Sampler State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
}; 

cbuffer cbPerDraw : register(b0)
{
	float4x4 tLAV: LAYERVIEW;
	float4x4 tV : VIEW;
	float4x4 tP: PROJECTION;
	float4x4 tLVP: LAYERVIEWPROJECTION;
};

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float4x4 tWV: WORLDLAYERVIEW;
	float4x4 tWIT: WORLDINVERSETRANSPOSE;
	
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float4x4 tColor <string uiname="Color Transform";>;
};

cbuffer cbTextureData : register(b2)
{
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
};

#include "PhongDirectional.fxh"

psInputTextured VS(float4 PosO: POSITION,
				   float3 NormO: NORMAL, 
				   float4 TexCd: TEXCOORD0,
				   float4 BlendWeights: BLENDWEIGHT,
				   float4 BlendIndices: BLENDINDICES)
{
    psInputTextured output;
	
	//---------- Skinning ----------
	float4 pos = 0;
	float3 norm = 0;

    for (int i = 0; i < 4; i++)
    {
        pos = pos + mul(PosO, JointMatrices[BlendIndices[i]]) * BlendWeights[i];
    	norm = norm + (mul(float4(NormO,0), JointMatrices[BlendIndices[i]]) * BlendWeights[i]).xyz;
    }
    //-------- End Skinning --------
  
    //inverse light direction in view space
    output.LightDirV = normalize(-mul(float4(lDir,0.0f), tV).xyz);
    
    //normal in view space
    output.NormV = normalize(mul(mul(norm, (float3x3)tWIT),(float3x3)tLAV).xyz);

    //position (projected)
    output.posScreen  = mul(pos, mul(tW, tLVP));
    output.uv = mul(TexCd, tTex);
    output.ViewDirV = -normalize(mul(pos, tWV).xyz);
    return output;
}

float4 PS(psInputTextured input): SV_Target
{
    float4 col = inputTexture.Sample(linearSampler, input.uv.xy);
    col.rgb = PhongDirectional(input.NormV, input.ViewDirV, input.LightDirV).xyz;
	col.a = 1;
    return col; //mul(col, tColor);
}

technique10 GouraudDirectional
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}